<?php

/*
 * 2007-2015 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author PrestaShop SA <contact@prestashop.com>
 *  @copyright  2007-2015 PrestaShop SA
 *  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * @since   1.0.1
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 *
 */
class epochta2w extends Module implements WidgetInterface {

    protected $_html = '';
    protected $default_wrap = 1;
    protected $templateFile;

    /** @var string */
    public static $currentIndex;
    public $debug_mode;
    public $token;
    public $public_api_key;
    public $private_api_key;
    public $request;

    public function __construct() {
        $this->name = 'epochta2w';
        $this->tab = 'administration';
        $this->version = '1.0.1';
        $this->author = 'ePochtaSMS.ru';
        $this->need_instance = 0;
        $this->secure_key = Tools::encrypt($this->name);
        $this->bootstrap = true;
        $this->debug_mode = (bool) Configuration::get('EP2W_API_DEBUG', 0);
        parent::__construct();
        $this->need_instance = 0;
        $this->public_api_key = Configuration::get('EP2W_API_PUBLIC_KEY');
        $this->private_api_key = Configuration::get('EP2W_API_PRIVATE_KEY');
        $this->displayName = $this->l('send_sms_epochta_name');
        $this->description = $this->l('send_sms_epochta_description');
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
        $this->tab_id = array('admin_cart_rules' => Tab::getIdFromClassName('AdminCartRules'));
        // $this->context = Context::getContext();
        if (isset($this->context->employee)) {
            $this->adminCartRulesToken = Tools::getAdminToken('AdminCartRules' . intval(Tab::getIdFromClassName('AdminCartRules')) . intval($this->context->employee->id));
            $this->adminCartsToken = Tools::getAdminToken('AdminCarts' . intval(Tab::getIdFromClassName('AdminCarts')) . intval($this->context->employee->id));
            $this->token = Tools::getAdminTokenLite('AdminModules');
        }
        // $this->context = Context::getContext();
        self::$currentIndex = 'index.php?controller=AdminModules&configure=epochta2w';
        $this->request = new Request($_GET, $_POST, array(), $_COOKIE, $_FILES, $_SERVER);
        require_once _PS_MODULE_DIR_ . '/epochta2w/libs/ProxyApi.php';
        
        // 
        $this->context->controller->addJS($this->_path.'js/jquery.typewatch.min.js', 'all');
    }
    
    public function hookHeader($param) {
    }

    public function directInstallHooks() {
        $this->installHooks();
    }

    public function renderWidget($hookName = null, array $configuration = []) {
        return '';
    }

    public function getWidgetVariables($hookName = null, array $configuration = []) {
        return '';
    }

    public function headerHTML() {
        return '';
    }

    public function getProductAttribute($id_product_old) {
        $res = Db::getInstance()->executeS('
                    SELECT `id_product_attribute`
                    FROM `' . _DB_PREFIX_ . 'product_supplier`
                    WHERE `id_product` = ' . (int) $id_product_old);

        $list_attr_id = array();
        foreach ($res as $item) {
            if (!empty($item['id_product_attribute'])) {
                $list_attr_id[] = $item['id_product_attribute'];
            }
        }
        return $list_attr_id;
    }

    /**
     * The method in which hooks are created for SMS sending rules
     * @return type
     */
    public function installHooks() {
        return $this->registerHook('actionAdminOrdersTrackingNumberUpdate') &&
                $this->registerHook('actionOrderReturn') &&
                $this->registerHook('actionOrderStatusPostUpdate') &&
                $this->registerHook('actionOrderStatusUpdate') &&
                $this->registerHook('actionPaymentConfirmation') &&
                $this->registerHook('actionPasswordRenew') &&
                $this->registerHook('actionValidateOrder') &&
                $this->registerHook('actionAuthentication') &&
                $this->registerHook('actionProductCancel');
    }

    /**
     * The method that is performed when the module is installed
     * 
     * @see Module::install()
     */
    public function install() {
        if (parent::install()  // Adds Module
                && $this->installHooks()) { // When installing the module, it creates hooks for sending SMS rules
            return true;
        }
        return false;
    }

    /**
     * Adds samples
     */
    // protected function installSamples() {}

    /**
     * The method that is executed when the module is removed
     * @see Module::uninstall()
     */
    public function uninstall() {
        // when deleting the module, you can do something, configure clean and send the grubby smile to the developer =)
        Hook::unregisterHook($this, 'actionAdminOrdersTrackingNumberUpdate');
        Hook::unregisterHook($this, 'actionOrderReturn');
        Hook::unregisterHook($this, 'actionOrderStatusPostUpdate');
        Hook::unregisterHook($this, 'actionOrderStatusUpdate');
        Hook::unregisterHook($this, 'actionPaymentConfirmation');
        Hook::unregisterHook($this, 'actionPasswordRenew');
        Hook::unregisterHook($this, 'actionValidateOrder');
        Hook::unregisterHook($this, 'actionAuthentication');
        Hook::unregisterHook($this, 'actionProductCancel');

        $res = true;

        // Deletes Module
        if (parent::uninstall()) {
            // Unsets configuration);
            $res &= Configuration::deleteByName('EP2W_API_DEBUG');
            $res &= Configuration::deleteByName('EP2W_API_MODE');
            $res &= Configuration::deleteByName('EP2W_RULES');
            $res &= Configuration::deleteByName('EP2W_API_PUBLIC_KEY');
            $res &= Configuration::deleteByName('EP2W_API_PRIVATE_KEY');
            return (bool) $res;
        }

        return false;
    }

    /**
     * Log of all events processed by the module
     * @param type $event
     */
    protected function execEvent($event) {
        $array_trace = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
        $str_trace = (new \Exception . '');
        if ($this->debug_mode) {
            PrestaShopLogger::addLog(__METHOD__ . ':' . $array_trace[1]['function'] . ': ' . $str_trace . print_r($event, 1));
        }
    }

    /**
     * Log checking rules
     * @return boolean
     */
    protected function checkRule($checkResult) {
        if ($this->debug_mode) {
            PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('checkResult'), 1));
        }
        return true;
    }

    /**
     * The sms result result list
     * @param type $event
     */
    protected function logResult($sendSmsResult) {
        if ($this->debug_mode) {
            PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('sendSmsResult'), 1));
        }
    }

    /**
     * The method of forming a hash of variables and their meanings
     * @param type $event
     * @return type
     */
    private function getReplaceMessageData($event) {
        $replace = array();
        if (isset($event['order'])) {
            $replace["##order.id##"] = $event['order']->id;
            $replace["##order.payment##"] = $event['order']->payment;
            $replace["##order.shipping_number##"] = $event['order']->shipping_number;
            if (!empty($event['order']->discount)) {
                $replace["##order.discount##"] = $event['order']->discount;
            }
            $replace["##order.total_paid##"] = $event['order']->total_paid;
            $replace["##order.id_currency##"] = $event['order']->id_currency;
            $replace["##order.delivery_number##"] = $event['order']->delivery_number;
            $replace["##order.date_add##"] = $event['order']->date_add;
            if (!empty($event['currency'])) {
                $replace["##order.currency.name##"] = $event['currency']->name;
                $replace["##order.currency.iso_code##"] = $event['currency']->iso_code;
                $replace["##order.currency.sign##"] = $event['currency']->sign;
            } else if (!empty($event['order']->id_currency)) {
                $currency = new Currency($event['order']->id_currency);
                if (!empty($currency)) {
                    $replace["##order.currency.name##"] = $currency->name;
                    $replace["##order.currency.iso_code##"] = $currency->iso_code;
                    $replace["##order.currency.sign##"] = $currency->sign;
                }
            }
        }
        if (isset($event['customer'])) {
            $replace["##customer.id##"] = $event['customer']->id;
            $replace["##customer.lastname##"] = $event['customer']->lastname;
            $replace["##customer.firstname##"] = $event['customer']->firstname;
            $replace["##customer.birthday##"] = $event['customer']->birthday;
            $replace["##customer.email##"] = $event['customer']->email;
            $replace["##customer.lastname##"] = $event['customer']->lastname;
        }
        if (isset($event['carrier'])) {
            $replace["##carrier.name##"] = $event['carrier']->name;
        }
        if (isset($event['password'])) {
            $replace["##password##"] = $event['password'];
        }
        // array for changing variables in message text
        if ($this->debug_mode) {
            PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('replace'), 1));
        }
        return $replace;
    }

    public static $mock_storage;

    /**
     * Get event data by name of event
     * @param type $name_event
     * @return type
     */
    public function getMockByEvent($name_event) {
        if (!isset(self::$mock_storage)) {
            /*
              self::$mock_storage = require_once ('/var/www/dump_hook'.ucfirst($name_event).'.php');
              $serialized_object = preg_replace('!s:(\d+):"(.*?)";!e', "'s:'.strlen('$2').':\"$2\";'", $serialized_object);
              $serialized_object = preg_replace_callback ( '!s:(\d+):"(.*?)";!', function($match) {
              return ($match[1] == strlen($match[2])) ? $match[0] : 's:' . strlen($match[2]) . ':"' . $match[2] . '";';
              }, $b_serialized_object );
             */
            $serialized_object = file_get_contents('/var/www/sdump_hook' . ucfirst($name_event) . '.php');
            self::$mock_storage = unserialize($serialized_object);
        }
        return self::$mock_storage;
    }

    /**
     * Testing hooks method
     * @param type $data_event
     * @param type $testData
     */
    public function testHookByMockObject($name_event) {
        $data_event = $this->getMockByEvent($name_event);
        switch ($name_event) {
            case 'actionAdminOrdersTrackingNumberUpdate':
                $this->hookActionAdminOrdersTrackingNumberUpdate($data_event);
                break;
            case 'actionOrderReturn':
                $this->hookActionOrderReturn($data_event);
                break;
            case 'actionProductCancel':
                $this->hookActionProductCancel($data_event);
                break;
            case 'actionOrderStatusPostUpdate':
                $this->hookActionOrderStatusPostUpdate($data_event);
                break;
            case 'actionOrderStatusUpdate':
                $this->hookActionOrderStatusUpdate($data_event);
                break;
            case 'actionPaymentConfirmation':
                $this->hookActionPaymentConfirmation($data_event);
                break;
            case 'actionPasswordRenew':
                $this->hookActionPasswordRenew($data_event);
                break;
            case 'actionValidateOrder':
                $this->hookActionValidateOrder($data_event);
                break;
            case 'actionAuthentication':
                $this->hookActionAuthentication($data_event);
                break;
        }
        exit;
    }

    protected function checkProducts($checkResult, $rule, $event) {
        try {
            // key 'product' change to some else
            if (!empty($rule['product']) AND ! empty($event['order'])) {
                $list_products = $event['order']->getProducts();
                foreach ($rule['product'] as $group) {
                    foreach ($group['product_rules'] as $attr) {
                        switch ($attr['type']) {
                            case 'products':
                                foreach ($list_products as $array_product) {
                                    if (in_array($array_product['product_id'], $attr['values'])) {
                                        $checkResult['products'] = true;
                                        break;
                                    }
                                }
                                break;
                            case 'attributes':
                                foreach ($list_products as $array_product) {
                                    $product = new Product($array_product['product_id']);
                                    $combinations = $product->getAttributeCombinations();
                                    $groups_attributes = array();
                                    foreach ($combinations as $comb) {
                                        $groups_attributes[$comb['id_product_attribute']][] = $comb['id_attribute'];
                                    }
                                    if (is_array($attr['values'])) {
                                        if (empty($groups_attributes)) {
                                            $checkResult['attributes'] = false;
                                            break;
                                        }
                                        foreach ($groups_attributes as $comb_attr) {
                                            $res = array_diff($attr['values'], $comb_attr);
                                            if (empty($res)) {
                                                $checkResult['attributes'] = true;
                                                break;
                                            }
                                        }
                                    }
                                }
                                break;
                            case 'categories':
                                foreach ($list_products as $array_product) {
                                    $product = new Product($array_product['product_id']);
                                    $array_category = $product->getCategories();
                                    if (is_array($attr['values'])) {
                                        if (empty($array_category)) {
                                            $checkResult['categories'] = false;
                                            break;
                                        }
                                        $res = array_diff($attr['values'], $array_category);
                                        if (empty($res)) {
                                            $checkResult['categories'] = true;
                                            break;
                                        }
                                    }
                                }
                                break;
                            case 'suppliers':
                                foreach ($list_products as $array_product) {
                                    // get suppliers
                                    $array_supplier = $this->getProductAttribute($array_product['product_id']);
                                    if (is_array($attr['values'])) {
                                        if (empty($array_supplier)) {
                                            $checkResult['suppliers'] = false;
                                            break;
                                        }
                                        $res = array_diff($attr['values'], $array_supplier);
                                        if (empty($res)) {
                                            $checkResult['suppliers'] = true;
                                            break;
                                        }
                                    }
                                }
                                break;
                            case 'manufacturers':
                                foreach ($list_products as $array_product) {
                                    if (in_array($array_product['id_manufacturer'], $attr['values'])) {
                                        $checkResult['manufacturers'] = true;
                                        break;
                                    }
                                }
                                break;
                        }
                    }
                }
            }
        } catch (Exception $e) {
            if ($this->debug_mode) {
                $str_trace = $e->__toString();
                PrestaShopLogger::addLog(__METHOD__ . ':' . $str_trace . print_r(compact('checkResult', 'rule', 'event'), 1));
            }
        }
        return $checkResult;
    }

    /**
     * Event 
     * @example go to order (example http://prestashop.local/admin4645rygov/index.php?controller=AdminOrders&token=c87a74fb6d2cf98f202b76db6ffaba65&vieworder&id_order=10)
     * shipping - edit
     * change tracking number and save
     */
    public function hookActionAdminOrdersTrackingNumberUpdate($event) {
        $this->execEvent($event);

        $rules = $this->getRules();
        $now = time();
        $replace = $this->getReplaceMessageData($event);

        // check for existing mobile phone numbers 
        if (!empty($event['order'])) {

            $id_address_delivery = $event['order']->id_address_delivery;
            $id_address_invoice = $event['order']->id_address_invoice;

            $address_delivery = new Address($id_address_delivery);
            $address_invoice = new Address($id_address_invoice);

            $phone_list = array();
            if (!empty($address_delivery->phone_mobile)) {
                $phone_list[] = $address_delivery->phone_mobile;
            }
            if (!empty($address_invoiactionAdminOrdersTrackingNumberUpdatece->phone_mobile)) {
                $phone_list[] = $address_invoice->phone_mobile;
            }
            if (empty($phone_list)) {
                return;
            }
        }

        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionAdminOrdersTrackingNumberUpdate') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }
                // check rule by carrier
                if (!empty($rule['carrier_select']) AND ! empty($event['carrier'])) {
                    $checkResult['carrier_select'] = in_array($event['carrier']->id_reference, $rule['carrier_select']);
                }
                // check rule by shop
                if (!empty($rule['shop_select']) AND ! empty($event['order'])) {
                    $checkResult['shop_select'] = in_array($event['order']->id_shop, $rule['shop_select']);
                }
                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);
                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                }
            }
        }
    }

    /**
     * Event
     * OrderFollowControllerCore::postProcess
     * @example login on user and move to http://prestashop.local/order-follow (create return on order with status delivered)
     * @param type $event
     */
    public function hookActionOrderReturn($event) {
        $this->execEvent($event);
        $rules = $this->getRules();
        $now = time();
        if (!empty($event['orderReturn']->id_order)) {
            $order = new Order($event['orderReturn']->id_order);
            $event['order'] = $order;
        }
        if (!empty($event['orderReturn']->id_customer)) {
            $customer = new Customer($event['orderReturn']->id_customer);
            $event['customer'] = $customer;
        }

        $replace = $this->getReplaceMessageData($event);

        // check for existing mobile phone numbers 
        if (!empty($event['order'])) {

            $id_address_delivery = $event['order']->id_address_delivery;
            $id_address_invoice = $event['order']->id_address_invoice;

            $address_delivery = new Address($id_address_delivery);
            $address_invoice = new Address($id_address_invoice);

            $phone_list = array();
            if (!empty($address_delivery->phone_mobile)) {
                $phone_list[] = $address_delivery->phone_mobile;
            }
            if (!empty($address_invoiactionAdminOrdersTrackingNumberUpdatece->phone_mobile)) {
                $phone_list[] = $address_invoice->phone_mobile;
            }
            if (empty($phone_list)) {
                return;
            }
        }

        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionOrderReturn') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }
                // check rule by carrier
                if (!empty($rule['carrier_select']) AND ! empty($event['carrier'])) {
                    $checkResult['carrier_select'] = in_array($event['carrier']->id_reference, $rule['carrier_select']);
                }
                // check rule by shop
                if (!empty($rule['shop_select']) AND ! empty($event['order'])) {
                    $checkResult['shop_select'] = in_array($event['order']->id_shop, $rule['shop_select']);
                }
                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);
                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                    $this->logResult($sendRes);
                }
            }
        }
    }

    /**
     * Event
     * actionProductCancel
     * @return type
     */
    public function hookActionProductCancel($event) {
        $this->execEvent($event);
        $rules = $this->getRules();
        $now = time();
        $order = new Order($event['id_order_detail']);
        $event['order'] = $order;

        if (!empty($order)) {
            $customer = new Customer($order->id_customer);
            if (!empty($customer)) {
                $event['customer'] = $customer;
            }
        }

        $replace = $this->getReplaceMessageData($event);

        // check for existing mobile phone numbers 
        if (!empty($event['order'])) {

            $id_address_delivery = $event['order']->id_address_delivery;
            $id_address_invoice = $event['order']->id_address_invoice;

            $address_delivery = new Address($id_address_delivery);
            $address_invoice = new Address($id_address_invoice);

            $phone_list = array();
            if (!empty($address_delivery->phone_mobile)) {
                $phone_list[] = $address_delivery->phone_mobile;
            }
            if (!empty($address_invoiactionAdminOrdersTrackingNumberUpdatece->phone_mobile)) {
                $phone_list[] = $address_invoice->phone_mobile;
            }
            
            if (empty($phone_list)) {
                return;
            }
        }
        
        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionProductCancel') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }
                // check rule by carrier
                if (!empty($rule['carrier_select']) AND ! empty($event['carrier'])) {
                    $checkResult['carrier_select'] = in_array($event['carrier']->id_reference, $rule['carrier_select']);
                }
                // check rule by shop
                if (!empty($rule['shop_select']) AND ! empty($event['order'])) {
                    $checkResult['shop_select'] = in_array($event['order']->id_shop, $rule['shop_select']);
                }
                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);
                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                    $this->logResult($sendRes);
                }
            }
        }
    }

    /**
     * Event
     * @example we come here http://prestashop.local/admin4645rygov/index.php?controller=AdminOrders&vieworder&token=c87a74fb6d2cf98f202b76db6ffaba65
     * choose status 11 or 12 and press to update the status
     * OrderHistoryCore::changeIdOrderState
     * @param type $event
     */
    public function hookActionOrderStatusPostUpdate($event) {
        $this->execEvent($event);
        $rules = $this->getRules();
        $now = time();
        $order = new Order($event['id_order']);
        $event['order'] = $order;

        if (!empty($order)) {
            $customer = new Customer($order->id_customer);
            if (!empty($customer)) {
                $event['customer'] = $customer;
            }
        }

        $replace = $this->getReplaceMessageData($event);

        // check for existing mobile phone numbers 
        if (!empty($event['order'])) {

            $id_address_delivery = $event['order']->id_address_delivery;
            $id_address_invoice = $event['order']->id_address_invoice;

            $address_delivery = new Address($id_address_delivery);
            $address_invoice = new Address($id_address_invoice);

            $phone_list = array();
            if (!empty($address_delivery->phone_mobile)) {
                $phone_list[] = $address_delivery->phone_mobile;
            }
            if (!empty($address_invoiactionAdminOrdersTrackingNumberUpdatece->phone_mobile)) {
                $phone_list[] = $address_invoice->phone_mobile;
            }
            if (empty($phone_list)) {
                return;
            }
        }

        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionOrderStatusPostUpdate') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }
                // check rule by carrier
                if (!empty($rule['carrier_select']) AND ! empty($event['carrier'])) {
                    $checkResult['carrier_select'] = in_array($event['carrier']->id_reference, $rule['carrier_select']);
                }
                // check rule by shop
                if (!empty($rule['shop_select']) AND ! empty($event['order'])) {
                    $checkResult['shop_select'] = in_array($event['order']->id_shop, $rule['shop_select']);
                }
                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);

                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                    $this->logResult($sendRes);
                }
            }
        }
    }

    /**
     * Event 
     * @example (Sets the new state of the given order)
     * OrderHistoryCore::changeIdOrderState
     * @param type $event
     */
    public function hookActionOrderStatusUpdate($event) {
        $this->execEvent($event);
        $rules = $this->getRules();
        $now = time();
        $order = new Order($event['id_order']);
        $event['order'] = $order;

        if (!empty($order)) {
            $customer = new Customer($order->id_customer);
            if (!empty($customer)) {
                $event['customer'] = $customer;
            }
        }
        $replace = $this->getReplaceMessageData($event);

        // check for existing mobile phone numbers 
        if (!empty($event['order'])) {

            $id_address_delivery = $event['order']->id_address_delivery;
            $id_address_invoice = $event['order']->id_address_invoice;

            $address_delivery = new Address($id_address_delivery);
            $address_invoice = new Address($id_address_invoice);

            $phone_list = array();
            if (!empty($address_delivery->phone_mobile)) {
                $phone_list[] = $address_delivery->phone_mobile;
            }
            if (!empty($address_invoiactionAdminOrdersTrackingNumberUpdatece->phone_mobile)) {
                $phone_list[] = $address_invoice->phone_mobile;
            }
            if (empty($phone_list)) {
                return;
            }
        }

        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionOrderStatusUpdate') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }
                // check rule by carrier
                if (!empty($rule['carrier_select']) AND ! empty($event['carrier'])) {
                    $checkResult['carrier_select'] = in_array($event['carrier']->id_reference, $rule['carrier_select']);
                }
                // check rule by shop
                if (!empty($rule['shop_select']) AND ! empty($event['order'])) {
                    $checkResult['shop_select'] = in_array($event['order']->id_shop, $rule['shop_select']);
                }
                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);
                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                    $this->logResult($sendRes);
                }
            }
        }
    }

    /**
     * Event
     * enter to (example: http://prestashop.local/admin4645rygov/index.php?controller=AdminOrders&token=c87a74fb6d2cf98f202b76db6ffaba65&vieworder&id_order=10)
     * update status
     * @param type $event
     */
    public function hookActionPaymentConfirmation($event) {
        $this->execEvent($event);
        $rules = $this->getRules();
        $now = time();
        $order = new Order($event['id_order']);
        $event['order'] = $order;

        if (!empty($order)) {
            $customer = new Customer($order->id_customer);
            if (!empty($customer)) {
                $event['customer'] = $customer;
            }
        }
        $replace = $this->getReplaceMessageData($event);

        // check for existing mobile phone numbers 
        if (!empty($event['order'])) {

            $id_address_delivery = $event['order']->id_address_delivery;
            $id_address_invoice = $event['order']->id_address_invoice;

            $address_delivery = new Address($id_address_delivery);
            $address_invoice = new Address($id_address_invoice);

            $phone_list = array();
            if (!empty($address_delivery->phone_mobile)) {
                $phone_list[] = $address_delivery->phone_mobile;
            }
            if (!empty($address_invoiactionAdminOrdersTrackingNumberUpdatece->phone_mobile)) {
                $phone_list[] = $address_invoice->phone_mobile;
            }
            if (empty($phone_list)) {
                return;
            }
        }

        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionPaymentConfirmation') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }
                // check rule by carrier
                if (!empty($rule['carrier_select']) AND ! empty($event['carrier'])) {
                    $checkResult['carrier_select'] = in_array($event['carrier']->id_reference, $rule['carrier_select']); // ???
                }
                // check rule by shop
                if (!empty($rule['shop_select']) AND ! empty($event['order'])) {
                    $checkResult['shop_select'] = in_array($event['order']->id_shop, $rule['shop_select']);
                }
                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);
                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                    $this->logResult($sendRes);
                }
            }
        }
    }

    /**
     * Event
     * PasswordControllerCore::changePassword
     * @param type $event
     */
    public function hookActionPasswordRenew($event) {
        $this->execEvent($event);
        $rules = $this->getRules();
        $now = time();
        $replace = $this->getReplaceMessageData($event);
        // check for existing mobile phone numbers 
        if (!empty($event['customer'])) {
            $lang = new Language((int) $this->context->language->id);
            $addresses = $event['customer']->getAddresses($lang->id);
            $phone_list = array();
            if (!empty($addresses)) {
                foreach ($addresses as $address) {
                    if (!empty($address['phone_mobile'])) {
                        $phone_list[] = $address['phone_mobile'];
                    }
                }
            }
            if (empty($phone_list)) {
                return;
            }
        }

        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionPasswordRenew') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }
                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);
                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                    $this->logResult($sendRes);
                }
            }
        }
    }

    /**
     * Event (Hook validate order)
     * PaymentModuleCore::validateOrder
     * @param type $event
     */
    public function hookActionValidateOrder($event) {
        $this->execEvent($event);
        $rules = $this->getRules();
        $now = time();
        $replace = $this->getReplaceMessageData($event);
        
        // var_dump($event);

        // check for existing mobile phone numbers 
        if (!empty($event['order'])) {

            $id_address_delivery = $event['order']->id_address_delivery;
            $id_address_invoice = $event['order']->id_address_invoice;

            $address_delivery = new Address($id_address_delivery);
            $address_invoice = new Address($id_address_invoice);

            $phone_list = array();
            if (!empty($address_delivery->phone_mobile)) {
                $phone_list[] = $address_delivery->phone_mobile;
            }
            if (!empty($address_invoiactionAdminOrdersTrackingNumberUpdatece->phone_mobile)) {
                $phone_list[] = $address_invoice->phone_mobile;
            }
            if (empty($phone_list)) {
                return;
            }
        }
        
        // var_dump($phone_list, $rules);
        // exit;

        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionValidateOrder') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by shop
                if (!empty($rule['shop_select']) AND ! empty($event['order'])) {
                    $checkResult['shop_select'] = in_array($event['order']->id_shop, $rule['shop_select']);
                }
                // check rule by shop
                if (!empty($rule['carrier_select']) AND ! empty($event['carrier'])) {
                    $checkResult['carrier_select'] = in_array($event['order']->id_carrier, $rule['carrier_select']);
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }
                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);
                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                    $this->logResult($sendRes);
                }
            }
        }
    }

    /**
     * Event 
     * @example http://prestashop.local/login?back=my-account (send form data user_group_test1@local.com/user_group_test1@local.com)
     * http://prestashop.local/?mylogout= - for logout
     * CustomerLoginFormCore::submit
     * @param type $event
     */
    public function hookActionAuthentication($event) {
        $this->execEvent($event);
        $rules = $this->getRules();
        $now = time();
        $replace = $this->getReplaceMessageData($event);

        // check for existing mobile phone numbers 
        if (!empty($event['customer'])) {
            $lang = new Language((int) $this->context->language->id);
            $addresses = $event['customer']->getAddresses($lang->id);
            $phone_list = array();
            if (!empty($addresses)) {
                foreach ($addresses as $address) {
                    if (!empty($address['phone_mobile'])) {
                        $phone_list[] = $address['phone_mobile'];
                    }
                }
            }
            if (empty($phone_list)) {
                return;
            }
        }

        foreach ($rules as $rule) {
            // check every rule
            if (!$rule['active']) {
                continue;
            }
            $checkResult = array();
            if ($rule['event_type'] === 'actionAuthentication') {
                // check rule by customer
                if (!empty($rule['id_customer']) AND ! empty($event['customer']->id)) {
                    $checkResult['customer'] = intval($rule['id_customer']) === $event['customer']->id;
                }
                // check rule by date from
                if (!empty($rule['date_from'])) {
                    $checkResult['date_from'] = strtotime($rule['date_from']) <= $now;
                }
                // check rule by date to
                if (!empty($rule['date_to'])) {
                    $checkResult['date_to'] = strtotime($rule['date_to']) >= $now;
                }

                $groups = Customer::getGroupsStatic((int) $event['customer']->id);
                // check rule by user's group
                if (!empty($rule['group_select'])) {
                    $groupsCompare = array_diff($rule['group_select'], $groups);
                    if (!count($groupsCompare)) {
                        $checkResult['group_select'] = true;
                    } else {
                        $checkResult['group_select'] = false;
                    }
                }
                $country = Customer::getCurrentCountry((int) $event['customer']->id);
                // check rule by user's address country
                if (!empty($rule['country_select'])) {
                    $checkResult['country_select'] = in_array($country, $rule['country_select']);
                }

                $checkResult = $this->checkProducts($checkResult, $rule, $event);

                $sender = $rule['sender'];
                $new_sender = null;
                if ($rule['sender_type'] == '2') {
                    $new_sender = $rule['new_sender'];
                }
                $message = strtr($rule['sms_body'], $replace);
                $date = ''; // $rule['date'];
                $translit = (int) $rule['translit'];

                $this->checkRule($checkResult);

                // success check rule
                if (!in_array(false, $checkResult, true)) {
                    $sendRes = $this->sendSms($message, $date, $phone_list, $translit, $sender, $new_sender);
                    $this->logResult($sendRes);
                }
            }
        }
    }

    /**
     * Translit text
     * @param type $str
     * @return string
     */
    public function translit($str) {
        if ($str) {
            $cirilica = array("а", "б", "в", "г", "д", "е", "ё", "з", "и", "й", "к", "л", "м", "н", "о", "п", "р", "с", "т", "у", "ф", "х", "ъ", "ы", "э", "_", "А", "Б", "В", "Г", "Д", "Е", "Ё", "З", "И", "Й", "К", "Л", "М", "Н", "О", "П", "Р", "С", "Т", "У", "Ф", "Х", "Ъ", "Ы", "Э", "_", "ж", "ц", "ч", "ш", "щ", "ь", "ю", "я", "Ж", "Ц", "Ч", "Ш", "Щ", "Ь", "Ю", "Я");
            $latinica = array("a", "b", "v", "g", "d", "e", "e", "z", "i", "y", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "f", "h", "", "i", "e", "_", "A", "B", "V", "G", "D", "E", "E", "Z", "I", "Y", "K", "L", "M", "N", "O", "P", "R", "S", "T", "U", "F", "H", "", "I", "E", "I", "zh", "ts", "ch", "sh", "shch", "", "yu", "ya", "ZH", "TS", "CH", "SH", "SHCH", " ", "YU", "YA");
            return str_replace($cirilica, $latinica, $str);
        } else {
            return '';
        }
    }

    /**
     * SMS sending method
     * @param type $message
     * @param type $date
     * @param type $telephone_list
     * @param type $translit
     * @param type $sender_id
     * @param type $new_sender
     * @return string
     */
    protected function sendSms($message, $date, $telephone_list, $translit, $sender_id, $new_sender = NULL) {
        // Here we look at the input data for sending messages        
        try {
            if ($this->debug_mode) {
                PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('message', 'date', 'telephone_list', 'translit', 'sender_id', 'new_sender'), 1));
            }
            $proxy_api = new ProxyApi();
            $proxy_api->connect($this->private_api_key, $this->public_api_key);
            $res = $proxy_api->addAddressbook($sender_id . '_date-' . $date . '_datetime-' . date('d-m-Y His'));
            if (isset($res["error"])) {
                throw new Exception($res["error"], $res['code']);
            } else {
                $book_id = $res["result"]["addressbook_id"];
            }

            if (isset($date)) {
                $timesend = $date;
            } else {
                $timesend = '';
            }

            if ($translit == 1) {
                $message = iconv("UTF-8", "UTF-8//IGNORE", $this->translit($message));
            }

            if (!empty($new_sender)) {
                $sender = $new_sender;
            } else {
                $res = $proxy_api->getSenderStatusById($sender_id);
                if (isset($res["error"])) {
                    throw new Exception($res["error"], $res['code']);
                } else {
                    $sender = $res["result"]["name"];
                }
            }

            if (isset($book_id)) {
                $success_count = 0;

                if (!$telephone_list) {
                    throw new Exception($this->l('Phones not found'));
                }

                $message_result = $error_format = '';
                foreach ($telephone_list as $telephone) {
                    list($phone, $first_last_name) = explode(';', $telephone);
                    if (!preg_match("/[3][8][0][0-9]{9}$/i", $phone)) {
                        $error_format .= $phone . __FILE__ . ':' . __LINE__ . "</br>";
                    }
                    $res = $proxy_api->addPhoneToAddressBook($book_id, $telephone, $first_last_name);
                    if (isset($res["error"])) {
                        $message_result .= 'Error: ' . $res["code"] . __FILE__ . ':' . __LINE__ . "<br>";
                    } else {
                        $success_count++;
                    }
                }
                if ($error_format != '') {
                    // $json['error']['format'] = 'Error phones: ' . __FILE__ . ':' . __LINE__ . $error_format;
                    // throw new Exception('Invalid phones: %s');
                }
                // if nothing add return error
                if (!$success_count) {
                    throw new Exception($this->l('Phones not found') . ' ' . $message_result);
                }
                $res = $proxy_api->createCampaign($sender, $message, $book_id, $timesend, 0, 0, 0);
                if (isset($res["error"])) {
                    throw new Exception($res["error"], $res['code']);
                } else {
                    $campaign_id = $res["result"]["id"];
                }

                if (isset($campaign_id)) {
                    $res = $proxy_api->getCampaignInfo($campaign_id);
                    if (isset($res["error"])) {
                        throw new Exception($res["error"], $res['code']);
                    } else {
                        $campinf_status_code = $res["result"]['status'];
                        $campinf_status = array(
                            0 => $this->l('In the sending queue'),
                            1 => $this->l('Not enough money'),
                            2 => $this->l('During the mailing process'),
                            3 => $this->l('Sent'),
                            4 => $this->l('There are no valid recipient numbers'),
                            5 => $this->l('Partially sent'),
                            6 => $this->l('Spam'),
                            7 => $this->l('Invalid sender name'),
                            8 => $this->l('Pause'),
                            9 => $this->l('Scheduled'),
                            10 => $this->l('Pending moderation'),
                            11 => $this->l('Duplicate mailing list'),
                            12 => $this->l('Disapproved'),
                        );
                        if (array_key_exists($campinf_status_code, $campinf_status)) {
                            $status = $campinf_status[$campinf_status_code];
                            $json['success'] = sprintf($this->l('text_success'), $status);
                        }
                    }
                }
            }
            // log the result of sending sms here
            if ($this->debug_mode) {
                PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('res', 'json'), 1));
            }
        } catch (Exception $e) {
            $code = $e->getCode();
            $detail_error = $this->getMessageByCode($code);
            $json['error']['message'] = $this->l('Error') . ": " . ($detail_error ? $detail_error : $e->getMessage()) . ($this->debug_mode ? (' at file ' . $e->getFile() . ' on line: ' . $e->getLine()) : '' ) . "<br>";
            $text_error = $e->__toString();
            PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('res', 'json', 'text_error'), 1));
        }

        if (!empty($book_id)) {
            $res = $proxy_api->delAddressbook($book_id);
        }
        return $json;
    }

    /**
     * Get customers addresses
     */
    public function ajaxProcessCustomerAddresses() {
        $res = array('found' => true);
        $addresses = $this->getCustomerAddresses();
        if (!empty($addresses)) {
            $res['addresses'] = $addresses;
        } else {
            $res = array('found' => false);
        }

        echo json_encode($res);
    }

    /**
     * Debug mysql queries, output ojecct data sql
     */
    public function getLastSqlQuery() {
        $db = Db::getInstance(_PS_USE_SQL_SLAVE_);
        var_dump($db);
    }

    /**
     * Search customers for ajax query
     * add to $this->content the result of Customer::SearchByName
     * (encoded in json)
     *
     * @return void
     */
    public function ajaxProcessSearchCustomers() {
        $customers = array();
        $send_type = (int) Tools::getValue('send_type');
        switch ($send_type) {
            case 1: // country
                $country_id = trim(Tools::getValue('country_id'));
                if (!empty($country_id) && $results = $this->getCustomerByCountry($country_id, 50)) {
                    foreach ($results as $result) {
                        if ($result['active']) {
                            $result['fullname_and_email'] = $result['firstname'] . ' ' . $result['lastname'] . ' - ' . $result['email'];
                            $customers[$result['id_customer']] = $result;
                        }
                    }
                }
                break;
            case 2: // gender
                $gender_id = trim(Tools::getValue('gender_id'));
                if (!empty($gender_id) && $results = $this->getCustomerByGender($gender_id, 50)) {
                    foreach ($results as $result) {
                        if ($result['active']) {
                            $result['fullname_and_email'] = $result['firstname'] . ' ' . $result['lastname'] . ' - ' . $result['email'];
                            $customers[$result['id_customer']] = $result;
                        }
                    }
                }
                break;
            case 3: // group
                $group_id = trim(Tools::getValue('group_id'));
                if (!empty($group_id) && $results = $this->getCustomerByGroup($group_id, 50)) {
                    foreach ($results as $result) {
                        //if ($result['active']) {
                        $result['fullname_and_email'] = $result['firstname'] . ' ' . $result['lastname'] . ' - ' . $result['email'];
                        $customers[$result['id_customer']] = $result;
                        //}
                    }
                }
                break;
            case 4: // name
                $searches = explode(' ', Tools::getValue('customer_search'));
                // allow split name by space (name1 name2 name3)
                $searches = array_unique($searches);
                foreach ($searches as $search) {
                    if (!empty($search) && $results = Customer::searchByName($search, 50)) {
                        foreach ($results as $result) {
                            if ($result['active']) {
                                $result['fullname_and_email'] = $result['firstname'] . ' ' . $result['lastname'] . ' - ' . $result['email'];
                                $customers[$result['id_customer']] = $result;
                            }
                        }
                    }
                }
                break;
        }

        if (count($customers) && Tools::getValue('sf2')) {
            $to_return = $customers;
        } elseif (count($customers) && !Tools::getValue('sf2')) {
            $to_return = array(
                'customers' => $customers,
                'found' => true
            );
        } else {
            $to_return = Tools::getValue('sf2') ? array() : array('found' => false);
        }
        echo json_encode($to_return);
    }

    /**
     * Send sms for ajax
     * add to $this->content the result of Customer::SearchByName
     * (encoded in json)
     *
     * @return void
     */
    public function ajaxProcessSendSms() {
        $sender_id = Tools::getValue('sender_id');
        $message = Tools::getValue('message');
        $send_date = Tools::getValue('send_date');
        $phones = Tools::getValue('phones');
        $translit = Tools::getValue('translit', 0);
        $new_sender = null;
        $sender_type = Tools::getValue('sender_type', 0);
        if ($sender_type == '2') {
            $new_sender = Tools::getValue('new_sender', 0);
        }
        $return = $this->sendSms($message, $send_date, $phones, $translit, $sender_id, $new_sender);
        echo json_encode($return);
    }

    /**
     * Search for customers by ip address.
     *
     * @param string $ip Searched string
     *
     * @return array|false|mysqli_result|null|PDOStatement|resource
     */
    public static function getCustomerByIpAddress($ip, $limit = 50) {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
        SELECT DISTINCT c.*
        FROM `' . _DB_PREFIX_ . 'customer` c
        LEFT JOIN `' . _DB_PREFIX_ . 'guest` g ON g.id_customer = c.id_customer
        LEFT JOIN `' . _DB_PREFIX_ . 'connections` co ON g.id_guest = co.id_guest 
        WHERE co.`ip_address` = \'' . (int) ip2long(trim($ip)) . '\' '
                        . ($limit ? 'LIMIT 0,' . (int) $limit : ''));
    }

    /**
     * Search for customers by country.
     *
     * @param string $ip Searched string
     *
     * @return array|false|mysqli_result|null|PDOStatement|resource
     */
    public static function getCustomerByCountry($id_country, $limit = 50) {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
        SELECT c.*
        FROM `' . _DB_PREFIX_ . 'customer` c
        LEFT JOIN `' . _DB_PREFIX_ . 'address` a ON a.id_customer = c.id_customer
        WHERE a.`id_country` = \'' . (int) $id_country . '\' '
                        . ($limit ? 'LIMIT 0,' . (int) $limit : ''));
    }

    /**
     * Search for customers by gender.
     *
     * @param string $ip Searched string
     *
     * @return array|false|mysqli_result|null|PDOStatement|resource
     */
    public static function getCustomerByGender($id_gender, $limit = 50) {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
        SELECT c.*
        FROM `' . _DB_PREFIX_ . 'customer` c
        WHERE c.`id_gender` = \'' . (int) $id_gender . '\' '
                        . ($limit ? 'LIMIT 0,' . (int) $limit : ''));
    }

    /**
     * Search for customers by group.
     *
     * @param string $ip Searched string
     *
     * @return array|false|mysqli_result|null|PDOStatement|resource
     */
    public static function getCustomerByGroup($group_id, $limit = 50) {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
        SELECT c.*
        FROM `' . _DB_PREFIX_ . 'customer` c
        LEFT JOIN `' . _DB_PREFIX_ . 'customer_group` g ON g.id_customer = c.id_customer
        WHERE g.`id_group` = \'' . (int) $group_id . '\' '
                        . ($limit ? 'LIMIT 0,' . (int) $limit : ''));
    }

    /**
     * Search for customers by list of id.
     *
     * @param string $ip Searched string
     *
     * @return array|false|mysqli_result|null|PDOStatement|resource
     */
    public static function getCustomerByListId($arr_id, $limit = 50) {
        $int_arr_id = array();
        foreach ($arr_id as $id) {
            $int_arr_id[] = (int) $id;
        }
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
        SELECT c.*
        FROM `' . _DB_PREFIX_ . 'customer` c
        WHERE c.`id_customer` IN (' . implode(',', $int_arr_id) . ') '
                        . ($limit ? 'LIMIT 0,' . (int) $limit : ''));
    }

    /**
     * Get gender list
     *
     * @param string $ip Searched string
     *
     * @return array|false|mysqli_result|null|PDOStatement|resource
     */
    public static function getGenderList() {
        $ob_genders = Gender::getGenders();
        $genders = array();
        foreach ($ob_genders as $gender) {
            $genders[] = array(
                'id' => $gender->id,
                'id_gender' => $gender->id_gender,
                'name' => $gender->name,
                'type' => $gender->type,
            );
        }
        return $genders;
    }

    /**
     * Output page content
     * 
     */
    public function getContent() {

        // if ($this->request->server->get('REQUEST_METHOD', '') === "POST") {}

        $this->_html .= $this->headerHTML();
        $action = Tools::getValue('action', null);

        switch ($action) {
            case 'test_event':
                $name_event = Tools::getValue('name_event');
                $this->testHookByMockObject($name_event);
                break;
            case 'send_sms':
                $senderId = Tools::getValue('sender');
                $message = Tools::getValue('sms_text');
                $translit = Tools::getValue('translit', 0);
                $date = Tools::getValue('send_date');
                $telephone_list = Tools::getValue('phones');
                $sms_phones = Tools::getValue('sms_phones');
                
                if ($telephone_list === false AND !empty($sms_phones)) {
                    $telephone_list = explode("\n", $sms_phones);
                }
                
                $this->sendSms($message, $date, $telephone_list, $translit, $senderId);
                break;
            case 'edit_settings':
                if (Tools::isSubmit('process')) {
                    Configuration::updateValue('EP2W_API_PUBLIC_KEY', Tools::getValue('public_api_key', ''));
                    Configuration::updateValue('EP2W_API_PRIVATE_KEY', Tools::getValue('private_api_key', ''));
                    // Configuration::updateValue('EP2W_API_MODE', (int) Tools::getValue('combat_mode', 0));
                    Configuration::updateValue('EP2W_API_DEBUG', (int) Tools::getValue('log_events', 0));
                    Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
                }
                break;
            case 'edit_rule':
                if (Tools::isSubmit('process')) {
                    $this->editProcess();
                }
                $rule_id = Tools::getValue('rule_id', null);
                if (isset($rule_id)) {
                    $rule_id = (int) $rule_id;
                }
                $this->_html .= $this->renderAddForm($rule_id);
                break;
            case 'add_rule':
                if (Tools::isSubmit('process')) {
                    $this->addProcess();
                }
                $this->_html .= $this->renderAddForm();
                break;
            case 'show_stat':
                if (Tools::isSubmit('process')) {
                    $this->postProcess();
                }
                $this->_html .= $this->renderStatTable();
                break;
            case 'delete_rule':
                $this->deleteProcess();
                // main action (main page)
                $display_info = 0;
                // try get info, in error, display error
                if (!empty($this->public_api_key)) {
                    $proxy_api = new ProxyApi();
                    $proxy_api->connect($this->private_api_key, $this->public_api_key);
                    $userInfo = $proxy_api->getUserBalance();
                    if (!isset($userInfo['error'])) {
                        // hide info and display info where user can do registration
                        $display_info = 1;
                    }
                }
                $this->_html .= $this->renderSettingsForm();
                if ($display_info) {
                    $this->_html .= $this->renderAccountInfo();
                    $this->_html .= $this->renderSendSmsForm();
                    $this->_html .= $this->renderList();
                }
                break;
            case 'edit_status':
                // add process
                $this->statusProcess();
                // main action (main page)
                $this->_html .= $this->renderSettingsForm();
                $this->_html .= $this->renderList();
                break;
            default :
                // MAIN page
                // add process
                if (Tools::isSubmit('process')) {
                    $this->postProcess();
                }
                // main action (main page)
                $display_info = 0;
                // try get info, in error, display error
                if (!empty($this->public_api_key)) {
                    $proxy_api = new ProxyApi();
                    $proxy_api->connect($this->private_api_key, $this->public_api_key);
                    $userInfo = $proxy_api->getUserBalance();
                    if (!isset($userInfo['error'])) {
                        // hide info and display info where user can do registration
                        $display_info = 1;
                    } else {
                        // TODO show error
                        $this->_html .= $this->displayError($userInfo['error']);
                    }
                }
                $this->_html .= $this->renderSettingsForm();
                if ($display_info) {
                    $this->_html .= $this->renderAccountInfo();
                    $this->_html .= $this->renderSendSmsForm();
                    $this->_html .= $this->renderList();
                }
                break;
        }

        return $this->_html;
    }

    protected function getRules() {
        $rules = json_decode(Configuration::get('EP2W_RULES', null, null, null, '[]'), 1);
        PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('rules'), 1));
        return $rules;
    }

    protected function prepareRule() {
        PrestaShopLogger::addLog(__METHOD__ . ':' . print_r($_POST, 1));
        $rule = array(
            'active' => Tools::getValue('status', 1),
            'sender' => Tools::getValue('sender', 1),
            'id_customer' => Tools::getValue('id_customer', 0),
            'event_type' => Tools::getValue('event_type', 0),
            'date_from' => Tools::getValue('date_from', 0),
            'date_to' => Tools::getValue('date_to', 0),
            'sms_body' => Tools::getValue('sms_body', 0),
            'translit' => Tools::getValue('translit', 0),
            'carrier_select' => Tools::getValue('carrier_select', array()),
            'group_select' => Tools::getValue('group_select', array()),
            'country_select' => Tools::getValue('country_select', array()),
            'shop_select' => Tools::getValue('shop_select', array()),
            'new_sender' => Tools::getValue('new_sender', ''),
            'sender_type' => Tools::getValue('sender_type', '0'),
        );

        if (is_array($ruleGroupArray = Tools::getValue('product_rule_group')) && count($ruleGroupArray)) {
            foreach ($ruleGroupArray as $ruleGroupId) {
                $rule['product'][$ruleGroupId]['id_product_rule_group'] = $ruleGroupId;
                $rule['product'][$ruleGroupId]['quantity'] = (int) Tools::getValue('product_rule_group_' . $ruleGroupId . '_quantity');
                $ruleArray = Tools::getValue('product_rule_' . $ruleGroupId);
                foreach ($ruleArray as $ruleId) {
                    $values = array();
                    foreach (Tools::getValue('product_rule_select_' . $ruleGroupId . '_' . $ruleId) as $id) {
                        $values[] = $id;
                    }
                    $values = array_unique($values);
                    $rule['product'][$ruleGroupId]['product_rules'][$ruleId] = array(
                        'type' => Tools::getValue('product_rule_' . $ruleGroupId . '_' . $ruleId . '_type'),
                        'values' => $values,
                    );
                }
            }
        }
        PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('rule'), 1));
        return $rule;
    }

    /**
     * Get rule
     * @param type $rule_id
     * @return boolean
     */
    protected function getRule($rule_id) {
        if (!isset($rule_id)) {
            return false;
        }

        $res = false;
        $rules = $this->getRules();
        if (isset($rules[$rule_id])) {
            $res = $rules[$rule_id];
        }
        PrestaShopLogger::addLog(__METHOD__ . ':' . print_r(compact('res'), 1));
        return $res;
    }

    /**
     * Set rule
     * @param array $rules
     * @return type
     */
    protected function setRules(array $rules) {
        $res = Configuration::updateValue('EP2W_RULES', json_encode($rules));
        return $res;
    }

    /**
     * Remove rule
     * @return boolean
     */
    public function deleteProcess() {
        $errors = array();
        /* Processes Rule */
        $rule_id = Tools::getValue('rule_id', null);
        if (!isset($rule_id)) {
            return false;
        }
        $rules = $this->getRules();
        if (isset($rules[$rule_id])) {
            unset($rules[$rule_id]);
            $res = $this->setRules($rules);
            $rules = json_decode($rules, 1);
            $this->clearCache();
            if (!$res) {
                $errors[] = $this->displayError($this->l('error_updated'));
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=6&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        }
    }

    public function clearCache() {
        
    }

    /**
     * Add new rule
     * 
     * @return boolean
     */
    protected function addProcess() {
        $errors = array();
        /* Processes Rule */
        if (Tools::isSubmit('process')) {
            $rule = $this->prepareRule();
            $rules = $this->getRules();
            $rules[] = $rule;
            $res = $this->setRules($rules);
            /* Update global shop context if needed */
            $this->clearCache();
            if (!$res) {
                $errors[] = $this->displayError($this->l('error_add'));
            }
        }
        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } else {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }

    protected function getSendSmsWarning() {
        return '<p class="alert alert-warning">' .
                $this->l('error_send_sms') .
                '</p>';
    }

    protected function getSendSmsSuccess() {
        return '<p class="alert alert-success">' .
                $this->l('success_send_sms') .
                '</p>';
    }

    /**
     * Edit rule
     * @return boolean
     */
    protected function editProcess() {
        $errors = array();
        /* Processes Rule */
        if (Tools::isSubmit('process')) {
            $rule_id = Tools::getValue('rule_id', null);
            if (!isset($rule_id)) {
                return false;
            }
            $rule = $this->prepareRule();
            $rules = $this->getRules();
            $rules[$rule_id] = $rule;

            $res = $this->setRules($rules);
            /* Update global shop context if needed */
            $this->clearCache();
            if (!$res) {
                $errors[] = $this->displayError($this->l('error_update'));
            }
        }
        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } else {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }

    /**
     * ??
     */
    protected function statusProcess() {
        $errors = array();
        /* Processes Rule */
        if (Tools::isSubmit('process')) {
            $rule_id = Tools::getValue('rule_id', null);
            $rule = $this->getRule($rule_id);
            $rule['active'] = Tools::getValue('status', 0);
            $rules = $this->getRules();
            $rules[$rule_id] = $rule;
            $res = $this->setRules($rules);
            $rules = json_decode($rules, 1);
            $this->clearCache();
            if (!$res) {
                $errors[] = $this->displayError($this->l('error_status'));
            }
        }
        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } else {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }

    /** @var int Number of results in list */
    public $listTotal = 0;

    /** @var array WHERE clause determined by filter fields */
    protected $_filter;

    /** @var array Number of results in list per page (used in select field) */
    public $_pagination = array(10, 20, 50, 100, 300, 1000);

    /** @var int Default number of results in list per page */
    public $_default_pagination = 50;

    /** @var string ORDER BY clause determined by field/arrows in list header */
    public $orderBy;

    /** @var string Default ORDER BY clause when $orderBy is not defined */
    public $_defaultOrderBy = false;

    /**
     * Output statistics sms compaign
     * 
     * @return type
     */
    public function renderStatTable() {
        // Pagination data. Determine total page number
        $pagination = $this->_default_pagination;
        if (in_array((int) Tools::getValue($this->list_id . '_pagination'), $this->_pagination)) {
            $pagination = (int) Tools::getValue($this->list_id . '_pagination');
        } elseif (isset($this->context->cookie->{$this->list_id . '_pagination'}) && $this->context->cookie->{$this->list_id . '_pagination'}) {
            $pagination = $this->context->cookie->{$this->list_id . '_pagination'};
        }

        $proxy_api = new ProxyApi();
        $proxy_api->connect($this->private_api_key, $this->public_api_key);

        $fields_display = array(
            'id' => array('title' => 'ID', 'align' => 'text-center', 'class' => 'fixed-width-xs',),
            'taskname' => array('title' => $this->l('stat name')),
            'from' => array('title' => $this->l('sender')),
            'body' => array('title' => $this->l('sms_text')),
            'begindate' => array('title' => $this->l('data_create'), 'align' => 'text-right', 'type' => 'datetime', 'filter_key' => 'a!date_add',),
            'enddate' => array('title' => $this->l('data_send_start'), 'align' => 'text-right', 'type' => 'datetime', 'filter_key' => 'a!date_add',),
            'status' => array(
                'title' => $this->l('status'),
                'type' => 'select',
                'color' => 'color',
                'list' => array(
                    0 => $this->l('In the sending queue'),
                    1 => $this->l('Not enough money'),
                    2 => $this->l('During the mailing process'),
                    3 => $this->l('Sent'),
                    4 => $this->l('There are no valid recipient numbers'),
                    5 => $this->l('Partially sent'),
                    6 => $this->l('Spam'),
                    7 => $this->l('Invalid sender name'),
                    8 => $this->l('Pause'),
                    9 => $this->l('Scheduled'),
                    10 => $this->l('Pending moderation'),
                    11 => $this->l('Duplicate mailing list'),
                    12 => $this->l('Disapproved'),
                ), 'filter_key' => 'os!id_order_state', 'filter_type' => 'int', 'order_key' => 'osname',
            ),
            'sent' => array('title' => $this->l('sent sms'), 'align' => 'text-center', 'class' => 'fixed-width-xs',),
            'delivered' => array('title' => $this->l('delivered sms'), 'align' => 'text-center', 'class' => 'fixed-width-xs',),
            'notdelivered' => array('title' => $this->l('not delivered sms'), 'align' => 'text-center', 'class' => 'fixed-width-xs',),
            // 'price' => array('title' => 'mailing cost', 'align' => 'text-right', 'type' => 'price', 'currency' => false), 
            'price' => array('title' => $this->l('mailing cost'), 'align' => 'text-center', 'class' => 'fixed-width-xs',),
            'currency' => array('title' => $this->l('Currency'), 'align' => 'text-center', 'class' => 'fixed-width-xs',),
        );

        $identifier = Tools::getIsset($this->identifier) ? '&' . $this->identifier . '=' . (int) Tools::getValue($this->identifier) : '';
        $order = '';
        if (Tools::getIsset($this->table . 'Orderby')) {
            $order = '&' . $this->table . 'Orderby=' . urlencode($this->orderBy) . '&' . $this->table . 'Orderway=' . urlencode(strtolower($this->orderWay));
        }

        $action = $this->currentIndex . $identifier . '&token=' . $token . '#' . $this->list_id;

        // Determine current page number
        $page = (int) Tools::getValue('submitFilter' . $this->list_id);

        if (!$page) {
            $page = 1;
        }

        // Choose number of results per page
        $selected_pagination = Tools::getValue(
                        $this->list_id . '_pagination', isset($this->context->cookie->{$this->list_id . '_pagination'}) ? $this->context->cookie->{$this->list_id . '_pagination'} : $this->_default_pagination
        );

        $list = [];
        $from = ($page - 1) * $selected_pagination;
        $res = $proxy_api->getCampaignList($from, $selected_pagination, 'desc');

        $this->listTotal = $res['result']['total'];

        $total_pages = max(1, ceil($this->listTotal / $pagination));

        if ($page > $total_pages) {
            $page = $total_pages;
        }
        $this->page = (int) $page;
        $status_color = array(
            0 => '#4169E1',
            1 => '#8f0621',
            2 => '#4169E1',
            3 => '#32CD32',
            4 => '#8f0621',
            5 => '#4169E1',
            6 => '#8f0621',
            7 => '#8f0621',
            8 => '#4169E1',
            9 => '#4169E1',
            10 => '#4169E1',
            11 => '#8f0621',
            12 => '#8f0621',
        );
        foreach ($res['result']['id'] as $i => $id) {
            $status = $res['result']['status'][$i];
            $list[] = array(
                'id' => $res['result']['id'][$i],
                'taskname' => $res['result']['taskname'][$i],
                'from' => $res['result']['from'][$i],
                'body' => $res['result']['body'][$i],
                'begindate' => $res['result']['begindate'][$i],
                'enddate' => $res['result']['enddate'][$i],
                'color' => $status_color[$status],
                'status' => $fields_display['status']['list'][$status],
                'sent' => $res['result']['sent'][$i],
                'delivered' => $res['result']['delivered'][$i],
                'notdelivered' => $res['result']['notdelivered'][$i],
                'price' => $res['result']['price'][$i],
                'currency' => $res['result']['currency'][$i]
            );
        }
        $view_data = array(
            'link' => $this->context->link,
            'slides' => $slides,
            'image_baseurl' => $this->_path . 'images/',
            'status_color' => $status_color,
            'page' => $page,
            'total_pages' => $total_pages,
            'selected_pagination' => $selected_pagination,
            'pagination' => $this->_pagination,
            'list_total' => $this->listTotal,
            'fields_display' => $fields_display,
            'list' => $list,
        );
        $this->context->smarty->assign($view_data);
        return $this->display(__FILE__, 'stat.tpl');
    }

    public function getSlides() {
        return array();
    }

    /**
     * ??
     * @return type
     */
    public function renderList() {
        $rules = $this->getRules();
        $proxy_api = new ProxyApi();
        $proxy_api->connect($this->private_api_key, $this->public_api_key);
        foreach ($rules as $i => $rule) {
            $res = $proxy_api->getSenderStatusById($rule['sender']);
            if (!isset($res["error"])) {
                $rules[$i]['sender_name'] = $res["result"]["name"];
            }
        }
        $this->context->smarty->assign(
                array(
                    'link' => $this->context->link,
                    'rules' => $rules,
                    'image_baseurl' => $this->_path . 'images/'
                )
        );
        return $this->display(__FILE__, 'list.tpl');
    }

    /**
     * Retrieve the cart rule product rule groups in the POST data
     * if available, and in the database if there is none
     *
     * @param CartRule $cart_rule
     *
     * @return array
     */
    public function getProductRuleGroupsDisplay($product_rule) {
        $product_rule_groups_array = array();
        $i = 1;
        foreach ($product_rule as $product_rule_group) {
            $j = 1;
            $product_rules_display = array();
            foreach ($product_rule_group['product_rules'] as $id_product_rule => $product_rule) {
                $product_rules_display[] = $this->getProductRuleDisplay($i, $j++, $product_rule['type'], $product_rule['values']);
            }
            $product_rule_groups_array[] = $this->getProductRuleGroupDisplay($i++, $product_rule_group['quantity'], $product_rules_display);
        }
        return $product_rule_groups_array;
    }

    /**
     * Return the form for a single cart rule group either with or without product_rules set up
     * 
     * @param type $product_rule_group_id
     * @param type $product_rule_group_quantity
     * @param type $product_rules
     * @return type
     */
    public function getProductRuleGroupDisplay($product_rule_group_id, $product_rule_group_quantity = 1, $product_rules = null) {
        Context::getContext()->smarty->assign('product_rule_group_id', $product_rule_group_id);
        Context::getContext()->smarty->assign('product_rule_group_quantity', $product_rule_group_quantity);
        Context::getContext()->smarty->assign('product_rules', $product_rules);
        return $this->createTemplate('product_rule_group.tpl')->fetch();
    }

    /**
     * Create a template from the override file, else from the base file.
     *
     * @param string $tpl_name filename
     * @return Smarty_Internal_Template
     */
    public function createTemplate($tpl_name) {
        return $this->context->smarty->createTemplate(_PS_MODULE_DIR_ . 'epochta2w/views/templates/admin/_configure/helpers/form/' . $tpl_name, $this->context->smarty);
    }

    public function getProductRuleDisplay($product_rule_group_id, $product_rule_id, $product_rule_type, $selected = array()) {
        Context::getContext()->smarty->assign(
                array(
                    'product_rule_group_id' => (int) $product_rule_group_id,
                    'product_rule_id' => (int) $product_rule_id,
                    'product_rule_type' => $product_rule_type,
                )
        );

        switch ($product_rule_type) {
            case 'attributes':
                $attributes = array('selected' => array(), 'unselected' => array());
                $results = Db::getInstance()->executeS('
				SELECT CONCAT(agl.name, " - ", al.name) as name, a.id_attribute as id
				FROM ' . _DB_PREFIX_ . 'attribute_group_lang agl
				LEFT JOIN ' . _DB_PREFIX_ . 'attribute a ON a.id_attribute_group = agl.id_attribute_group
				LEFT JOIN ' . _DB_PREFIX_ . 'attribute_lang al ON (a.id_attribute = al.id_attribute AND al.id_lang = ' . (int) Context::getContext()->language->id . ')
				WHERE agl.id_lang = ' . (int) Context::getContext()->language->id . '
				ORDER BY agl.name, al.name');
                foreach ($results as $row) {
                    $attributes[in_array($row['id'], $selected) ? 'selected' : 'unselected'][] = $row;
                }
                Context::getContext()->smarty->assign('product_rule_itemlist', $attributes);
                $choose_content = $this->createTemplate('product_rule_itemlist.tpl')->fetch();
                Context::getContext()->smarty->assign('product_rule_choose_content', $choose_content);
                break;
            case 'products':
                $products = array('selected' => array(), 'unselected' => array());
                $results = Db::getInstance()->executeS('
				SELECT DISTINCT name, p.id_product as id
				FROM ' . _DB_PREFIX_ . 'product p
				LEFT JOIN `' . _DB_PREFIX_ . 'product_lang` pl
					ON (p.`id_product` = pl.`id_product`
					AND pl.`id_lang` = ' . (int) Context::getContext()->language->id . Shop::addSqlRestrictionOnLang('pl') . ')
				' . Shop::addSqlAssociation('product', 'p') . '
				WHERE id_lang = ' . (int) Context::getContext()->language->id . '
				ORDER BY name');
                foreach ($results as $row) {
                    $products[in_array($row['id'], $selected) ? 'selected' : 'unselected'][] = $row;
                }
                Context::getContext()->smarty->assign('product_rule_itemlist', $products);
                $choose_content = $this->createTemplate('product_rule_itemlist.tpl')->fetch();
                Context::getContext()->smarty->assign('product_rule_choose_content', $choose_content);
                break;
            case 'manufacturers':
                $products = array('selected' => array(), 'unselected' => array());
                $results = Db::getInstance()->executeS('
				SELECT name, id_manufacturer as id
				FROM ' . _DB_PREFIX_ . 'manufacturer
				ORDER BY name');
                foreach ($results as $row) {
                    $products[in_array($row['id'], $selected) ? 'selected' : 'unselected'][] = $row;
                }
                Context::getContext()->smarty->assign('product_rule_itemlist', $products);
                $choose_content = $this->createTemplate('product_rule_itemlist.tpl')->fetch();
                Context::getContext()->smarty->assign('product_rule_choose_content', $choose_content);
                break;
            case 'suppliers':
                $products = array('selected' => array(), 'unselected' => array());
                $results = Db::getInstance()->executeS('
				SELECT name, id_supplier as id
				FROM ' . _DB_PREFIX_ . 'supplier
				ORDER BY name');
                foreach ($results as $row) {
                    $products[in_array($row['id'], $selected) ? 'selected' : 'unselected'][] = $row;
                }
                Context::getContext()->smarty->assign('product_rule_itemlist', $products);
                $choose_content = $this->createTemplate('product_rule_itemlist.tpl')->fetch();
                Context::getContext()->smarty->assign('product_rule_choose_content', $choose_content);
                break;
            case 'categories':
                $categories = array('selected' => array(), 'unselected' => array());
                $results = Db::getInstance()->executeS('
				SELECT DISTINCT name, c.id_category as id
				FROM ' . _DB_PREFIX_ . 'category c
				LEFT JOIN `' . _DB_PREFIX_ . 'category_lang` cl
					ON (c.`id_category` = cl.`id_category`
					AND cl.`id_lang` = ' . (int) Context::getContext()->language->id . Shop::addSqlRestrictionOnLang('cl') . ')
				' . Shop::addSqlAssociation('category', 'c') . '
				WHERE id_lang = ' . (int) Context::getContext()->language->id . '
				ORDER BY name');
                foreach ($results as $row) {
                    $categories[in_array($row['id'], $selected) ? 'selected' : 'unselected'][] = $row;
                }
                Context::getContext()->smarty->assign('product_rule_itemlist', $categories);
                $choose_content = $this->createTemplate('product_rule_itemlist.tpl')->fetch();
                Context::getContext()->smarty->assign('product_rule_choose_content', $choose_content);
                break;
            default :
                Context::getContext()->smarty->assign('product_rule_itemlist', array('selected' => array(), 'unselected' => array()));
                Context::getContext()->smarty->assign('product_rule_choose_content', '');
        }

        return $this->createTemplate('product_rule.tpl')->fetch();
    }

    /**
     * Show form add new link
     * 
     * @param type $rule_id
     * @return type
     */
    public function renderAddForm($rule_id) {
        $fexamplelist = 0;
        $type_rule_list[] = array('id' => 1, 'name' => 'actionAdminOrdersTrackingNumberUpdate');
        $type_rule_list[] = array('id' => 1, 'name' => 'actionOrderReturn');
        $type_rule_list[] = array('id' => 1, 'name' => 'actionOrderCancel');
        $type_rule_list[] = array('id' => 1, 'name' => 'actionOrderStatusPostUpdate'); // after
        $type_rule_list[] = array('id' => 1, 'name' => 'actionOrderStatusUpdate'); // before
        $type_rule_list[] = array('id' => 1, 'name' => 'actionPaymentConfirmation');
        $type_rule_list[] = array('id' => 1, 'name' => 'actionPasswordRenew');
        $type_rule_list[] = array('id' => 1, 'name' => 'actionProductOutOfStock'); // not found where called ?
        $type_rule_list[] = array('id' => 1, 'name' => 'actionValidateOrder');
        $type_rule_list[] = array('id' => 1, 'name' => 'actionAuthentication');
        // $type_rule_list[] = array('id' => 1, 'name' => 'displayPaymentReturn'); // realy ?
        $type_rule_list[] = array('id' => 1, 'name' => 'actionPaymentConfirmation');
        $type_rule_list[] = array('id' => 1, 'name' => 'displayPaymentReturn'); // realy ?

        $proxy_api = new ProxyApi();
        $proxy_api->connect($this->private_api_key, $this->public_api_key);
        $senders = $proxy_api->getSenderStatusAll();
        $sender_list = array();
        foreach ($senders['result']['data'] as $item) {
            $sender_list[] = array('id' => $item[0], 'name' => $item[2]);
        }

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Information'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'hint' => '11',
                        'type' => 'select',
                        'label' => $this->l('type of rule'),
                        'desc' => 'desc',
                        'name' => 'type_rule',
                        'required' => true,
                        'default_value' => 0,
                        'options' => array(
                            'query' => $type_rule_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'hint' => '11',
                        'type' => 'select',
                        'col' => '4',
                        'label' => $this->l('senders'),
                        'desc' => 'desc',
                        'name' => 'sender',
                        'required' => true,
                        'default_value' => (int) $this->context->country->id,
                        'options' => array(
                            'query' => $sender_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => 'Sms text',
                        'name' => 'description',
                        'autoload_rte' => true,
                        'lang' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->getTranslator()->trans('Enabled', array(), 'Admin.Global'),
                        'name' => 'active_slide',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->getTranslator()->trans('Save', array(), 'Admin.Actions'),
                )
            ),
        );

        $customer = false;
        if (isset($rule_id)) {
            // load rule
            $rule = $this->getRule($rule_id);
            if (!empty($rule['id_customer'])) {
                // All the filter are prefilled with the correct information
                $customer_filter = '';
                if (//Validate::isUnsignedId($rule['id_customer']) &&
                        ($customer = new Customer($rule['id_customer'])) &&
                        Validate::isLoadedObject($customer)) {
                    $customer_filter = $customer->firstname . ' ' . $customer->lastname . ' (' . $customer->email . ')';
                }
            }
        }

        $cart_rule = new CartRule();
        $currencies = Currency::getCurrencies(false, true, true);
        $countries = $cart_rule->getAssociatedRestrictions('country', true, true);
        $countries['unselected'] = Country::getCountries($this->context->language->id);
        $countries['selected'] = array();

        if (!empty($countries['unselected'])) {
            foreach ($countries['unselected'] as $i => $country) {
                if (in_array($country['id_country'], $rule['country_select'])) {
                    unset($countries['unselected'][$i]);
                    $countries['selected'][] = $country;
                }
            }
        }

        $groups = $cart_rule->getAssociatedRestrictions('group', false, true);
        $groups['unselected'] = array_merge($groups['selected'], $groups['unselected']);
        $groups['selected'] = array();

        if (!empty($groups['unselected'])) {
            foreach ($groups['unselected'] as $i => $group) {
                if (in_array($group['id_group'], $rule['group_select'])) {
                    unset($groups['unselected'][$i]);
                    $groups['selected'][] = $group;
                }
            }
        }

        $shops = $cart_rule->getAssociatedRestrictions('shop', false, false);
        $shops['unselected'] = array_merge($shops['selected'], $shops['unselected']);
        $shops['selected'] = array();

        if (!empty($shops['unselected'])) {
            foreach ($shops['unselected'] as $i => $shop) {
                if (in_array($shop['id_shop'], $rule['shop_select'])) {
                    unset($shops['unselected'][$i]);
                    $shops['selected'][] = $shop;
                }
            }
        }

        $carriers = $cart_rule->getAssociatedRestrictions('carrier', true, false);
        $carriers['unselected'] = array_merge($carriers['selected'], $carriers['unselected']);
        $carriers['selected'] = array();

        if (!empty($carriers['unselected'])) {
            foreach ($carriers['unselected'] as $i => $carrier) {
                if (in_array($carrier['id_carrier'], $rule['carrier_select'])) {
                    unset($carriers['unselected'][$i]);
                    $carriers['selected'][] = $carrier;
                }
            }
        }
        $product_rule_groups = $this->getProductRuleGroupsDisplay($rule['product']);
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) $this->context->language->id);
        $helper->default_form_language = $lang->id;
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitRule';
        $helper->currentIndex = 'index.php?controller=AdminModules';
        $helper->token = $this->token;
        $language = new Language((int) $this->context->language->id);
        $helper->tpl_vars = array(
            'action' => Tools::getValue('action'),
            'base_url' => $this->context->shop->getBaseURL(),
            'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'image_baseurl' => $this->_path . 'images/',
            'currentTab' => $this,
            'cart_rule' => $cart_rule,
            'currencies' => $currencies,
            'countries' => $countries,
            // 'customerFilter' => $customer_filter,
            'carriers' => $carriers,
            'currentIndex' => self::$currentIndex,
            'product_rule_groups' => $product_rule_groups,
            'product_rule_groups_counter' => count($product_rule_groups),
            'groups' => $groups,
            'shops' => $shops,
            'rule_id' => $rule_id,
            'customer' => $customer,
            'rule' => $rule,
            'sender_list' => $sender_list,
            'event_vars' => array(
                'actionAdminOrdersTrackingNumberUpdate' => array(
                    'order.id' => 6,
                    'order.payment' => $this->l('Payment by check'),
                    'order.shipping_number' => '111222335',
                    'order.discount' => '0.000000',
                    'order.total_paid' => '9088.360000',
                    'order.id_currency' => '1',
                    'order.currency.name' => $this->l('Russian ruble'),
                    'order.currency.iso_code' => 'RUB',
                    'order.currency.sign' => $this->l('currency.rub'),
                    'order.delivery_number' => '0',
                    'order.date_add' => '2017-09-11 15:46:23',
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                ),
                'actionAuthentication' => array(
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                ),
                'actionOrderReturn' => array(
                    'order.id' => 6,
                    'order.payment' => $this->l('Payment by check'),
                    'order.shipping_number' => '111222335',
                    'order.discount' => '0.000000',
                    'order.total_paid' => '9088.360000',
                    'order.id_currency' => '1',
                    'order.currency.name' => $this->l('Russian ruble'),
                    'order.currency.iso_code' => $this->l('Russian ruble'),
                    'order.currency.sign' => $this->l('currency.rub'),
                    'order.delivery_number' => '0',
                    'order.date_add' => '2017-09-11 15:46:23',
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                ),
                'actionOrderStatusPostUpdate' => array(
                    'order.id' => 6,
                    'order.payment' => $this->l('Payment by check'),
                    'order.shipping_number' => '111222335',
                    'order.discount' => '0.000000',
                    'order.total_paid' => '9088.360000',
                    'order.id_currency' => '1',
                    'order.currency.name' => $this->l('Russian ruble'),
                    'order.currency.iso_code' => 'RUB',
                    'order.currency.sign' => $this->l('currency.rub'),
                    'order.delivery_number' => '0',
                    'order.date_add' => '2017-09-11 15:46:23',
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                ),
                'actionOrderStatusUpdate' => array(
                    'order.id' => 6,
                    'order.payment' => $this->l('Payment by check'),
                    'order.shipping_number' => '111222335',
                    'order.discount' => '0.000000',
                    'order.total_paid' => '9088.360000',
                    'order.id_currency' => '1',
                    'order.currency.name' => $this->l('Russian ruble'),
                    'order.currency.iso_code' => 'RUB',
                    'order.currency.sign' => $this->l('currency.rub'),
                    'order.delivery_number' => '0',
                    'order.date_add' => '2017-09-11 15:46:23',
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                ),
                'actionPasswordRenew' => array(
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                    'password' => '111111',
                ),
                'actionPaymentConfirmation' => array(
                    'order.id' => 6,
                    'order.payment' => $this->l('Payment by check'),
                    'order.shipping_number' => '111222335',
                    'order.discount' => '0.000000',
                    'order.total_paid' => '9088.360000',
                    'order.id_currency' => '1',
                    'order.currency.name' => $this->l('Russian ruble'),
                    'order.currency.iso_code' => 'RUB',
                    'order.currency.sign' => $this->l('currency.rub'),
                    'order.delivery_number' => '0',
                    'order.date_add' => '2017-09-11 15:46:23',
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                ),
                'actionProductCancel' => array(
                    'order.id' => 6,
                    'order.payment' => $this->l('Payment by check'),
                    'order.shipping_number' => '111222335',
                    'order.discount' => '0.000000',
                    'order.total_paid' => '9088.360000',
                    'order.id_currency' => '1',
                    'order.currency.name' => $this->l('Russian ruble'),
                    'order.currency.iso_code' => 'RUB',
                    'order.currency.sign' => $this->l('currency.rub'),
                    'order.delivery_number' => '0',
                    'order.date_add' => '2017-09-11 15:46:23',
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                ),
                'actionValidateOrder' => array(
                    'order.id' => 6,
                    'order.payment' => $this->l('Payment by check'),
                    'order.shipping_number' => '111222335',
                    'order.discount' => '0.000000',
                    'order.total_paid' => '9088.360000',
                    'order.id_currency' => '1',
                    'order.currency.name' => $this->l('Russian ruble'),
                    'order.currency.iso_code' => 'RUB',
                    'order.currency.sign' => $this->l('currency.rub'),
                    'order.delivery_number' => '0',
                    'order.date_add' => '2017-09-11 15:46:23',
                    'customer.id' => '2',
                    'customer.lastname' => 'Ivan',
                    'customer.firstname' => 'Ivan',
                    'customer.birthday' => '0000-00-00',
                    'customer.email' => 'presta2@shop.local',
                    'carrier.name' => 'My carrier',
                )
            ),
            'currentToken' => $this->token,
            'adminCartsToken' => $this->adminCartsToken,
            'adminCartRulesToken' => $this->adminCartRulesToken,
            'module_dir' => _PS_MODULE_DIR_,
        );
        $helper->override_folder = '/';
        $this->fields_form = array();
        $languages = Language::getLanguages(false);

        if (count($languages) > 1) {
            return $this->getMultiLanguageInfoMsg() . $helper->generateForm(array($fields_form));
        } else {
            return $helper->generateForm(array($fields_form));
        }
    }

    /**
     * Output account information by api data
     * @return type
     */
    public function renderAccountInfo() {
        $proxy_api = new ProxyApi();
        $proxy_api->connect($this->private_api_key, $this->public_api_key);
        $userInfo = $proxy_api->getUserBalance();
        $viewData = array(
            'balance' => $userInfo['result'],
        );
        $this->context->smarty->assign($viewData);
        return $this->display(__FILE__, 'account_info.tpl');
    }

    public function getMessageByCode($code) {
        switch ($code) {
            case '201':
                return $this->l('sms_error_201');
                break;
            case '202':
                return $this->l('sms_error_202');
                break;
            case '203':
                return $this->l('sms_error_203');
                break;
            case '211':
                return $this->l('sms_error_211');
                break;
            case '212':
                return $this->l('sms_error_212');
                break;
            case '213':
                return $this->l('sms_error_213');
                break;
            case '218':
                return $this->l('sms_error_218');
                break;
            case '264':
                return $this->l('sms_error_264');
                break;
            case '266':
                return $this->l('sms_error_266');
                break;
            case '267':
                return $this->l('sms_error_267');
                break;
            case '300':
                return $this->l('sms_error_300');
                break;
            case '301':
                return $this->l('sms_error_301');
                break;
            case '302':
                return $this->l('sms_error_302');
                break;
            case '303':
                return $this->l('sms_error_303');
                break;
            case '304':
                return $this->l('sms_error_304');
                break;
            case '305':
                return $this->l('sms_error_305');
                break;
            case '306':
                return $this->l('sms_error_306');
                break;
            case '310':
                return $this->l('sms_error_310');
                break;
            case '311':
                return $this->l('sms_error_311');
                break;
        }
    }

    /**
     * Output form for manual send sms
     * @return type
     */
    public function renderSendSmsForm() {
        $proxy_api = new ProxyApi();
        $proxy_api->connect($this->private_api_key, $this->public_api_key);
        $senders = $proxy_api->getSenderStatusAll();

        $sender_list = array();
        foreach ($senders['result']['data'] as $item) {
            $sender_list[] = array('id' => $item[0], 'name' => $item[2]);
        }

        $fields_form = array(
            'table' => '1',
            'form' => array(
                'id_form' => 'step_carrier_general',
                'legend' => array(
                    'title' => $this->l('Send Sms'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'customer',
                        'name' => 'customer',
                    ),
                    array(
                        'type' => 'hidden',
                        'name' => 'action',
                    ),
                    array(
                        'type' => 'select',
                        'col' => '4',
                        'label' => $this->l('send_sms_sender'),
                        'desc' => $this->l('send_sms_sender'),
                        'name' => 'sender',
                        'required' => true,
                        'default_value' => (int) $this->context->country->id,
                        'options' => array(
                            'query' => $sender_list,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'hint' => '11',
                        'required' => true,
                        'type' => 'textarea',
                        'label' => $this->l('sms text'),
                        'name' => 'sms_text',
                        'autoload_rte' => false,
                        // 'lang' => true,
                        'placeholder' => "380933630000; Var1; Var2",
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Transliteration'),
                        'name' => 'translit',
                        'desc' => $this->l('Transliteration text before send'),
                        'values' => array(
                            array(
                                'id' => 'test',
                                'value' => 1,
                                'label' => $this->l('yes'),
                            ),
                            array(
                                'id' => 'work',
                                'value' => 0,
                                'label' => $this->l('no')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Immediately send sms'),
                        'name' => 'immediately_send_sms',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'yes',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ),
                            array(
                                'id' => 'no',
                                'value' => 0,
                                'label' => $this->l('No')
                            )
                        ),
                    ),
                    array(
                        'type' => 'datetime',
                        'label' => $this->l('Send date'),
                        'name' => 'send_date',
                        'id' => 'send_date',
                        'lang' => false,
                        // 'required' => true, 
                        'size' => 10,
                    ),
                ),
                'buttons' => array(
                    'save_and_preview' => array(
                        'name' => 'viewcms',
                        'href' => $this->context->link->getAdminLink('AdminModules', false) . '&token=' . $this->token . '&configure=' . $this->name . '&module_name=' . $this->name . '&action=show_stat',
                        'type' => 'submit',
                        'title' => $this->l('Preview stats'),
                        'class' => 'btn btn-default pull-left',
                        'icon' => 'process-icon-preview'
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Send'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->base_folder = _PS_MODULE_DIR_ . 'epochta2w/views/templates/admin/_configure/helpers/form/';
        $helper->base_tpl = 'send_sms.tpl';
        $helper->show_toolbar = false;
        $helper->table = 'send_sms';
        $lang = new Language((int) $this->context->language->id);
        $helper->default_form_language = $lang->id;
        // $this->fields_form = array();

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'process';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = $this->token;

        $lang_id = Context::getContext()->language->id;
        $genders = $this->getGenderList();
        $groups = Group::getGroups($lang_id, true);

        $helper->tpl_vars = array(
            'fields_value' => $this->getSendSmsFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'currentTab' => $this,
            'link' => $this->context->link,
            'genders' => $genders,
            'groups' => $groups,
            'countries' => Country::getCountries($this->context->language->id),
        );
        return $helper->generateForm(array($fields_form));
    }

    /**
     * Set form default fields values for send sms
     * @return type
     */
    protected function getSendSmsFieldsValues() {
        $field = array(
            'action' => 'send_sms',
            'private_api_key' => Configuration::get('EP2W_API_PRIVATE_KEY'),
        );
        return $field;
    }

    /**
     * Output module config form
     * 
     * @return type
     */
    public function renderSettingsForm() {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->getTranslator()->trans('Settings', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs'
                ),
                'description' => $this->l('reg_for_api_key') . '<a href="https://atomic.center/registration/index/services/sms/">https://atomic.center</a>',
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'action',
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('public_api_key'),
                        'required' => true,
                        'name' => 'public_api_key',
                        'desc' => $this->l('public_api_key')
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('private_api_key'),
                        'required' => true,
                        'name' => 'private_api_key',
                        'desc' => $this->l('private_api_key'),
                    ),
                    /*array(
                        'type' => 'switch',
                        'label' => $this->l('Api work mode'),
                        'name' => 'combat_mode',
                        'desc' => $this->l('api mode: work or test'),
                        'values' => array(
                            array(
                                'id' => 'test',
                                'value' => 1,
                                'label' => $this->l('yes')
                            ),
                            array(
                                'id' => 'work',
                                'value' => 0,
                                'label' => $this->l('no')
                            )
                        ),
                    ),*/
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Log events'),
                        'name' => 'log_events',
                        'desc' => $this->l('Log events desc'),
                        'values' => array(
                            array(
                                'id' => 'test',
                                'value' => 1,
                                'label' => $this->l('yes')
                            ),
                            array(
                                'id' => 'work',
                                'value' => 0,
                                'label' => $this->l('no')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) $this->context->language->id);
        $helper->default_form_language = $lang->id;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'process';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = $this->token;

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'currentTab' => $this,
        );
        return $helper->generateForm(array($fields_form));
    }

    /**
     * Return field value if possible (both classical and multilingual fields)
     *
     * Case 1 : Return value if present in $_POST / $_GET
     * Case 2 : Return object value
     *
     * @param ObjectModel $obj Object
     * @param string $key Field name
     * @param int|null $id_lang Language id (optional)
     * @return string
     */
    public function getFieldValue($obj, $key, $id_lang = null, $default_value = null) {
        if ($id_lang) {
            if (is_object($obj)) {
                $default_value = (isset($obj->id) && $obj->id && isset($obj->{$key}[$id_lang])) ? $obj->{$key}[$id_lang] : false;
            } elseif (is_array($obj)) {
                $default_value = (isset($obj['id']) && $obj['id'] && isset($obj[$key][$id_lang])) ? $obj[$key][$id_lang] : false;
            }
        } else {
            if (is_object($obj)) {
                $default_value = isset($obj->{$key}) ? $obj->{$key} : false;
            } elseif (is_array($obj)) {
                $default_value = isset($obj[$key]) ? $obj[$key] : false;
            }
        }
        return Tools::getValue($key . ($id_lang ? '_' . $id_lang : ''), $default_value);
    }

    /**
     * Set default form fields value for module config form
     * @return type
     */
    public function getConfigFieldsValues() {
        return array(
            'action' => 'edit_settings',
            'public_api_key' => Configuration::get('EP2W_API_PUBLIC_KEY'),
            'private_api_key' => Configuration::get('EP2W_API_PRIVATE_KEY'),
            // 'combat_mode' => (int) Configuration::get('EP2W_API_MODE', 0),
            'log_events' => (int) Configuration::get('EP2W_API_DEBUG', 0),
        );
    }

    /**
     * Get customer addressed by lang
     * @return type
     */
    public function getCustomerAddresses() {
        $customer = new Customer(Tools::getValue('id_customer'));
        $lang = new Language((int) $this->context->language->id);
        return $customer->getAddresses($lang->id);
    }

}

?>
