<?php

class APISMS {

    private $privateKey;
    private $publicKey;
    private $formatResponse;
    private $url;
    private $version;
    private $testMode;
    private $type;
    
    private $lastResponse;

    public function getLastResponse() {
        return $this->lastResponse;
    }

    function __construct($privateKey, $publicKey, $url, $testMode = false, $version = '3.0', $formatResponse = 'json') {
        $this->privateKey = $privateKey;
        $this->publicKey = $publicKey;
        $this->formatResponse = $formatResponse;
        $this->url = $url;
        $this->version = $version;
        $this->testMode = $testMode;
    }

    /**
     * Form request to API service
     *
     * @param $path
     * @param string $method
     * @param array $data
     * @param bool $useToken
     * @return array|NULL
     */
    public function send($url, $method = 'GET', $data = array(), $headers = array()) {
        $curlLoaded = extension_loaded('curl');
        if ( ! $curlLoaded) {
            $this->type = 'simple';
        } else {
            $this->type = 'curl';
        }
        switch ($this->type) {
            case 'simple':
                if (!ini_get('allow_url_fopen'))
                    throw new \Exception("Please install curl or enable allow_url_fopen in your php settings.");
                switch ($method) {
                    case 'GET':
                        $http = array(
                            'method' => $method,
                            'content' => http_build_query($data),
                            'timeout' => 10, // 10 sec
                        );
                        if (!empty($data)) {
                            $url .= '?' . http_build_query($data);
                        }
                        break;
                    default:
                        $http = array(
                            'method' => $method,
                            'content' => http_build_query($data),
                        );
                        break;
                }
                if (!empty($headers)) {
                    $strHeaders = '';
                    foreach ($headers as $value) {
                        $strHeaders .= $value . PHP_EOL;
                    }
                    $http['header'] = $strHeaders;
                }
                $context = stream_context_create(array(
                    'http' => $http
                ));
                $responseBody = file_get_contents($file = $url, $use_include_path = false, $context);
                $pattern = '/^HTTP\/[0-9]\.[0-9] ([0-9]{3}) (.*)$/';
                $headerString = $http_response_header[0];
                if (preg_match($pattern, $headerString, $match)) {
                    $headerCode = (int) $match[1];
                }                
                break;
            case 'curl':
                $curl = curl_init();
                if (!empty($headers)) {
                    curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
                }
                /**
                 *  curl_setopt($cURL,CURLOPT_HTTPHEADER,array (
                  "Content-Type: text/xml; charset=utf-8",
                  "Expect: 100-continue"
                  ));
                 */
                // curl_setopt( $ch, CURLOPT_ENCODING, "UTF-8" );
                switch ($method) {
                    case 'POST':
                        curl_setopt($curl, CURLOPT_POST, count($data));
                        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));
                        break;
                    case 'PUT':
                        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "PUT");
                        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));
                        break;
                    case 'DELETE':
                        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'DELETE');
                        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));
                        break;
                    default:
                        if (!empty($data)) {
                            $url .= '?' . http_build_query($data);
                        }
                }
                curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10); // 10 sec
                curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
                curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
                curl_setopt($curl, CURLOPT_URL, $url);
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($curl, CURLOPT_HEADER, 1);
                curl_setopt($curl, CURLINFO_HEADER_OUT, 1);
                $response = curl_exec($curl);
                $header_size = curl_getinfo($curl, CURLINFO_HEADER_SIZE);
                $headerCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                $responseBody = substr($response, $header_size);
                if (curl_errno($curl) > 0) {
                    $errno = curl_errno($curl);
                    $errorMess = curl_error($curl);
                }
                curl_close($curl);
                break;
        }

        $result = array($headerCode, $responseBody, compact('errno', 'errorMess'));

        $this->lastResponse = $result;

        return $result;
    }

    public function execCommad($command, $params, $simple = false) {
        $params['key'] = $this->publicKey;
        if ($this->testMode)
            $params['test'] = true;
        $controlSUM = $this->calcControlSum($params, $command);
        $params['sum'] = $controlSUM;

        //$curlLoaded = extension_loaded('curl');

        //var_dump($curlLoaded);
        //exit;

        /*if ($this->type == 'curl'
                AND ! $curlLoaded) {
            $this->type = 'simple';
        }*/

        /*$ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_URL, $this->url . $this->version . '/' . $command);
        $result = curl_exec($ch);
        */
        
        try {
            
            // var_dump($command);
        
            $result = $this->send($this->url . $this->version . '/' . $command, 'POST', $params);
            
            file_put_contents($_SERVER['DOCUMENT_ROOT'].'/log_c.txt.', $command . "\n", LOCK_EX | FILE_APPEND);

            
            if ($command == 'sendSms') {
                
                
                // echo print_r($result, 1);

            //    exit;
            }

            if ($result[2]['errno'] > 0)
                return array('success' => false, 'code' => $result[2]['errno'], 'error' => $result[2]['errorMess']);
            elseif ($this->formatResponse == 'json')
                return $this->processResponseJSON($result[1], $simple);
            elseif ($this->formatResponse == 'xml')
                return $this->processResponseXML($result[1]);
            else
                return $this->processResponseJSON($result[1]);
        
        } catch (\Exception $e) {
            return array('success' => false, 'code' => $e->getCode(), 'error' => $e->getMessage());
        }
        // var_dump($result);

        // var_dump($result);


    }

    private function processResponseJSON($result, $simple) {
        if ($simple)
            return json_decode($result, true);
        elseif ($result) {
            $jsonObj = json_decode($result, true);
            if (null === $jsonObj) {
                return array('success' => false, 'result' => NULL);
            } elseif (!empty($jsonObj->error)) {
                return array('success' => false, 'error' => $jsonObj->error, 'code' => $jsonObj->code);
            } else {
                return $jsonObj;
            }
        } else {
            return array('success' => false, 'result' => NULL);
        }
    }

    private function processResponseXML($result, $simple) {
        //:TODO processResponseXML
        return NULL;
    }

    private function calcControlSum($params, $action) {
        $params['version'] = $this->version;
        $params['action'] = $action;
        ksort($params);
        $sum = '';
        foreach ($params as $k => $v)
            $sum .= $v;
        $sum .= $this->privateKey;
        return md5($sum);
    }

}
