<?php

require_once (__DIR__ .'/config.php');
require_once (__DIR__ .'/Addressbook.php');
require_once (__DIR__ .'/Exceptions.php');
require_once (__DIR__ .'/Account.php');
require_once (__DIR__ .'/Stat.php');

class ProxyApi {
    
    private $api_gateway;
    private $api_stat;
    private $api_account;
    private $api_addressbook;
    
    private $privateKey;
    private $publicKey;
    private $url = 'http://atompark.com/api/sms/';
    
    public function connect($privateKey,$publicKey,$testMode=false) {
        if ( ! empty($privateKey) AND ! empty($publicKey)) {
            $this->api_gateway = new APISMS($privateKey,$publicKey,$this->url,$testMode);
            $this->api_stat = new Stat($this->api_gateway);
            $this->api_account = new Account($this->api_gateway);
            $this->api_addressbook = new Addressbook($this->api_gateway);
        } else 
            throw new Exception('Error api connect, check settings');
    }

    public function setMode($testMode=false) {
        $this->api_gateway = new APISMS($this->privateKey,$this->publicKey,$this->url,$testMode);
        $this->api_stat = new Stat($this->api_gateway);
        $this->api_account = new Account($this->api_gateway);
        $this->api_addressbook = new Addressbook($this->api_gateway);
    }
    
    // API
    
    public function getUserBalance($currency = null) {
        return $this->api_account->getUserBalance($currency);
    }
    
    public function addAddressbook($name, $description = NULL) {
        return $this->api_addressbook->addAddressbook($name);
    }
    
     public function addPhoneToAddressBook($book_id, $telephone, $variables) {
        return $this->api_addressbook->addPhoneToAddressBook($book_id, $telephone, $variables);
    }
    
     public function createCampaign($senderName, $message, $book_id, $timesend) {
        return $this->api_stat->createCampaign($senderName, $message, $book_id, $timesend, 0, 0, 0);
    }
    
     public function getCampaignInfo($campaign_id) {
        return $this->api_stat->getCampaignInfo($campaign_id);
    }
    
     public function delAddressbook($book_id) {
        return $this->api_addressbook->delAddressbook($book_id);
    }

    public function getCampaignList($from, $offset, $direction) {
        return $this->api_stat->getCampaignList($from, $offset, $direction);
    }
    
    public function getCampaignListExt($from, $limit, $orderfield, $direction, $search) {
        return $this->api_stat->getCampaignListExt($from, $limit, $orderfield, $direction, $search);
    }
    
    public function sendSMS($sender, $text, $phone, $datetime, $sms_lifetime) {
        return $this->api_stat->sendSMS($sender, $text, $phone, $datetime, $sms_lifetime);
    }
    
    /**
     * Get Sender objects.
     * Return status:
     * 0-moderation
     * 1-registered
     * 2-rejected
     */
    public function getSenderStatusAll($from=NULL,$offset=NULL){
        return $this->api_account->getSenderStatusAll($from,$offset);
    }
    
    public function getSenderStatusById($id) {
        return $this->api_account->getSenderStatusById($id);
    }
}

?>
